"""Custom steps for login tests."""

import random
import string

from behave import given, when
from behave_webdriver.steps import *


def before_all(context):
    pass  # login and save cookies here

@when(u'I open the nextcloud URL')
@given(u'I open the nextcloud URL')
def step_impl(context):
    """Open nextcloud URL."""
    context.behave_driver.get(context.nextcloud['url'])


@when(u'I open the grafana URL')
@given(u'I open the grafana URL')
def step_impl(context):
    """Open grafana URL."""
    context.behave_driver.get(context.grafana['url'])


@when(u'I enter the "{section}" "{cred_type}" in the inputfield "{element}"')
def step_impl(context, section, cred_type, element):
    """Enter username/password into login inputfields."""
    elem = context.behave_driver.get_element(element)
    elem.clear()

    context_section = getattr(context, section)
    value = context_section[cred_type]
    elem.send_keys(value)

@when(u'I add a random string to the inputfield "{element}"')
def step_impl(context, element):
    """Add a random string of 20 characters to the specified input field"""
    random_string = ''.join(random.choices(string.ascii_letters, k=20))
    context.execute_steps(
        'When I add "{}" to the inputfield "{}"'.format(
            random_string, element))

@when(u'I close the nextcloud welcome wizard if it exists')
def step_impl(context):
    """Checks if the #firstrunwizard element exists and closes it if so"""
    if context.behave_driver.element_exists("#firstrunwizard"):
        context.execute_steps(
            'When I click on the element "a.close.icon-close"')
